<?php
/**
* Plugin Name: WooCommerce QDL
* Plugin URI: https://www.qdl.sk/?plugin
* Description: Plugin pre správu balíkov prepravnej služby QDL
* Version: 5.0
* Author: QDL.sk
* Author URI: https://www.qdl.sk
* Text Domain: QDL
*
* WC requires at least: 4.4
* WC tested up to: 5.2.1
*
* Potřebuje: PHP 5.3.0+
*
* Používáné META tagy v objednávce:
* _qdl_tracking_code - Sledovací číslo balíku, oddělené středníkem pokud více
* _qdl_id - ID order
* _qdl_pickup_order (bool) - Zda byl pro balík objednán svoz
* _qdl_pickup_date (Y-m-d) - Datum, kdy se balík importoval
* _qdl_packages_weight - Seznam hmotností v Kg odděleno +, pokud více zásilek
* _qdl_driver_note - Poznámka řidiči
* _qdl_shippingPickup_code - Vybraný kód pobočky
*/


 
if ( ! defined( 'ABSPATH' ) ) {
    exit;
} // Exit if accessed directly


if (!in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
	return; // woocommerce neni aktivni
}


include_once __DIR__.'/functions.php';


// HPOS kompatibilní
add_action( 'before_woocommerce_init', function() {
	if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
		\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
	}
});




//---------------- DETAIL OBJEDNÁVKY - META BOX
function qdl_add_custom_box()
{
	$screen = class_exists( '\Automattic\WooCommerce\Internal\DataStores\Orders\CustomOrdersTableController' ) && wc_get_container()->get( \Automattic\WooCommerce\Internal\DataStores\Orders\CustomOrdersTableController::class )->custom_orders_table_usage_is_enabled()
		? wc_get_page_screen_id( 'shop-order' )
		: 'shop_order';
	
	$post_id = get_the_ID();
	if (!qdl_is($post_id)) return; // nejedná se o QDL objednávku
	
	add_meta_box(
		'qdl_box',           // Unique ID
		'QDL',  // Box title
		'qdl_box_html',  // Content callback, must be of type callable
		$screen                  // Post type
	);
}
add_action('add_meta_boxes', 'qdl_add_custom_box');



/**
* STORNO ZÁSILKY ----------->>> TY HOOKS PRO NOTICES NEJSOU ÚPLNĚ IDEÁLNÍ PODLE MĚ
*/
function qdl_saved_order_items($post_id) // při uložení objednávky
{
	if (!empty($_POST['qdl_cancel']))
	{
		$notices = qdl_deleteShipment($post_id);
		
		update_option('qdl_notices', $notices);
	}
}
add_action('woocommerce_saved_order_items', 'qdl_saved_order_items');


function qdl_delete_order($post_id) // v případě smazání objednávky
{
	qdl_deleteShipment($post_id);
}
add_action('woocommerce_delete_order', 'qdl_delete_order');

		
		
function qdl_deleteShipment($post_id)
{
	$ord = wc_get_order($post_id);
	
	$trackingCodes = qdl_getTrackingCodes($post_id);
	

	// zrušíme META TAGY
	qdl_delete_post_meta($post_id, '_qdl_tracking_code');
	qdl_delete_post_meta($post_id, '_qdl_pickup_date');
	qdl_delete_post_meta($post_id, '_qdl_pickup_order');
	
	
	if (empty($trackingCodes)) return; // není co mazat
	
	$deleted = true;
	$notices = '';
	
	// ZÁSILKU NELZE STORNOVAT PŘES API
	$notices.= '<div class="notice notice-error is-dismissible"><p><b>Podacie čislo bolo z objednávky vymazané. Zásielku č. '.implode(", ", $trackingCodes).' je potrebné stornovať v <a href="'.qdl_editUrl($post_id).'" target="_blank">informačnom systéme QDL</a>.</b></p><script>window.open(\''.qdl_editUrl($post_id).'\')</script></div>';
	
	// přidáme poznámku k objednávce
	$message = sprintf( __( 'Stornované zásielky č. '.implode(", ", $trackingCodes), 'my-textdomain' ), wp_get_current_user()->display_name);
	$ord->add_order_note( $message );
	
	return $notices;
}



/**
* ADMIN NOTICES -- toto není optimální, ale nemůžu dohledat jak to udělat 100%
*/
function qdl_admin_notice() {
	$notices = get_option('qdl_notices');
	if (!empty($notices)) {
		echo $notices;
		delete_option('qdl_notices');
	}
}
add_action('admin_notices', 'qdl_admin_notice');




/**
* PANEL v detailu objednávky
*/
function qdl_box_html($post)
{
	$post_id = $post->ID;
    $pickupDate = qdl_get_post_meta($post_id, '_qdl_pickup_date', true);
	
	$dir = plugins_url('qdl');
	
	echo '<style>
.qdlpanel{display: flex;  text-align: center;    margin: auto;}
.qdlpanel .button {display: flex;line-height: 1.5;white-space: pre-wrap;justify-content: center;height: auto;max-height: initial;align-content: center; align-items: center;}
.qdlpanel .button+.button { margin-left: 5px; }
.qdlpanel-rows { margin: 12px 1px 0 1px;}
.qdlpanel-rows>A.label{ cursor:pointer; }
.qdlpanel-rows>.label { text-decoration:none; color:#000; display: flex; cursor: default; flex: 1; border-bottom: 1px solid #ddd; width: 100%; box-sizing: border-box; border-radius: 3px; padding: 8px 0; align-items: center; }
.qdlpanel-rows>.label>SPAN:first-child { flex-grow: 0; flex-shrink: 1; }
.qdlpanel-rows>.label>SPAN:last-child { padding-left: 8px; }
.qdlpanel-rows .label:last-child { border-bottom: 0; }
</style>';	
	
	$codes = qdl_getTrackingCodes($post_id);
	$trackingCodes = implode(";", $codes);
	
	$wasImported = count($codes) > 0;
	
	echo '<div data-qdl-ajaxable="mainbox">'; //--- ajaxable obsah
	
	// KONTROLNÍ PANEL
	echo '<div class="qdlpanel">';
	if ($wasImported) {
		// ZÁSILKA BYLA IMPORTOVÁNA
		echo '<a href="'.$dir.'/popup.php?action=labels&codes='.urlencode($trackingCodes).'" onclick="var pos=prompt(\'Pozícia etikety? (1-4)\', 1);if(pos!==null) qdl_popWin(this.href+\'&pos=\'+pos);return false;" class="button" style="width:10%;min-width:80px">Tlač štítku</a>';
		echo '<a href="'.$dir.'/popup.php?action=pickuplist&pickup_date='.urlencode($pickupDate).'" onclick="qdl_popWin(this.href);return false;" class="button">Podací hárok '.date('d.m.Y', strtotime($pickupDate)).'</a>';
		echo '<button type="submit" name="qdl_cancel" value="1" onclick="if(!confirm(\'POZOR! Zásielku je nutné stornovať v systéme QDL.sk\nTýmto iba odstránite podacie číslo u tejto objednávky, aby ste mohli vykonať opätovný import. Pokračovať?\')) return false;" class="button" style="width:10%;min-width:70px">Stornovať zásielku</button>';
	}
	else {
		echo '<a href="'.$dir.'/popup.php?action=import&post_ids='.$post_id.'" onclick="var pos=prompt(\'Pozícia etikety? (1-4)\', 1);if(pos!==null) qdl_popWin(this.href+\'&pos=\'+pos);return false;" class="button">Importovať do QDL a tlač štítku</a>';
		// echo '<a href="'.$dir.'/popup.php?action=create_pickup" onclick="qdl_popWin(this.href);return false;" class="button">Objednat svoz</a>';		
	}
	echo '</div>';
	
	
	
	echo '<div class="qdlpanel-rows" id="post-'.$post_id.'">'; // potřebujeme ID kvůli JS ukládání
	
	if ($wasImported) {
		$url = qdl_trackingLink($codes);
		
		echo '<label class="label">';
		if (count($codes) > 1) {
			$str = 'Číslo zásielok';
		} else {
			$str = 'Číslo zásielky';
		}
		echo '<span>'.$str.':</span><span><a href="'.$url.'" target="_blank">'.implode(", ", $codes).'</a></span>';
		echo '</label>';
	}
	
	
	$defaultWeight = qdl_getConfig('default_weight');
	$hasDefaultWeight = true;
	if (!is_numeric($defaultWeight)) {
		$defaultWeight = '0';
		$hasDefaultWeight = false;
	}
	
	
	$weight = qdl_get_post_meta($post->ID, '_qdl_packages_weight', true);
	echo '<label class="label">';
	$weights = $weight ? explode("+", $weight) : [];
	$str = 'Zadané hmotnosti';
	echo '<span>'.$str.':</span>';
	echo '<span>';
	if ($wasImported) {
		echo '<b>'.implode(" + ", $weights).' kg</b>';
	} else {
		echo ' ';
		// ukládáme přes AJAX (onblur event)
		// potřebujeme znát hmotnosti (potažmo počet balíků)
		$weights = qdl_get_post_meta($post_id, '_qdl_packages_weight', true);
		echo '<input type="text" data-name="qdl_packages_weight" class="tips" data-tip="U viac balíkov píšte viac hmotnosťou 1 + 5 + 2 alebo 2,5 a 2"';
		echo ' value="'.esc_attr($weights).'" placeholder="'.$defaultWeight.'" style="width:65px;';
		if ($weights == '' && !$hasDefaultWeight) echo 'background:#ffeded;'; // @note: nutno změnit taky v JS
		echo '"> kg';
	}
	echo'</span>';
	echo '</label>';
	
	
	// _qdl_driver_note
	echo '<label class="label">';
	echo '<span>Poznámka vodiči:</span><span>';
	$note = qdl_get_post_meta($post_id, '_qdl_driver_note', true);
	if ($wasImported)  {
		echo $note ? $note : '<b>ne</b>';
	} else {
		echo '<button type="button" class="button tips qdl_driver_note" data-tip="Poznámka vodiči" data-value="'.esc_attr($note).'">';
		echo '<i class="fa'.(empty($note) ? 'r' : '').' fa-sticky-note"></i>'; // @note: nutno i v JS
		echo '</button>';
	}	
	echo '</span>';
	echo '</label>';
	
	echo '</div>';
	
	
	echo '</div>'; //--- konec ajaxable
}



/**
* TOHLE PŘIDÁME DO DETAILŮ OBJEDNÁVKY
*/
function qdl_after_customer($wc_order)
{
	$post_id = $wc_order->get_id();
	
	$trackingCodes = qdl_getTrackingCodes($post_id);
	if ($trackingCodes)
	{
		// PŘIDÁME SLEDOVACÍ ČÍSLO
		echo '<h2 class="woocommerce-column__title">Číslo pre sledovanie zásielky</h2>';
		echo '<p>Zásielka č. <a href="'.qdl_trackingLink($trackingCodes).'"> (QDL)';
		echo implode(", ", $trackingCodes);
		echo '</a>';
	}
}

// u konce objednávky (checkout)
add_action('woocommerce_order_details_before_order_table', 'qdl_after_customer');

// email s novou objednávkou
add_action('woocommerce_email_customer_details', 'qdl_after_customer');


// Registrovať shortcode pre zobrazovanie sledovacieho čísla objednávky
function custom_order_tracking_shortcode() {
    $post_id = get_the_ID();
    if (qdl_is($post_id)) {
        $_qdl_tracking_code = qdl_get_post_meta($post_id, '_qdl_tracking_code', true);
        if (empty($_qdl_tracking_code)) {
            return '';
        }
		$qdl_link = qdl_trackingLink($_qdl_tracking_code);
        return '<p>Sledovacie číslo objednávky: <a href="' . $qdl_link . '">' . $_qdl_tracking_code . '</a></p>';
    }
}
add_shortcode('qdl_tracking', 'custom_order_tracking_shortcode');



/**
* SLOUPEC "QDL" V OBJEDNÁVKÁCH
*/
function qdl_columns($columns)
{
	if (!is_array($columns)) {
		$columns = (array) $columns;
	}
	
	$mykA = 'order_qdl_shipping';
	$myvA = 'Doprava';
	
	$myk = 'order_qdl no-link';
	$myv = 'QDL';
	
	$newcolumns = array();
	foreach ($columns as $k => $v)
	{
		if ($k == 'order_total' && !isset($newcolumns[$myk])) {
			// vložíme před
			$newcolumns[$mykA] = $myvA;
			$newcolumns[$myk] = $myv;
			unset($mykA);
		}
		
		$newcolumns[$k] = $v;
	}
	
	if (isset($mykA)) {
		// no tak vložíme nakonec
		$newcolumns[$mykA] = $myvA;
		$newcolumns[$myk] = $myv;
	}
	
	return $newcolumns;	
}


/**
* OBSAH SLOUPCE
*/
function qdl_column_values($column, $order = null)
{
	if (is_object($order) && is_numeric($order->id)) {
		$post_id = $order->id;
	} else {
		global $post;
		$post_id = $post->ID;
	}
	
	if ($column == 'order_qdl_shipping')
	{
		$ord = wc_get_order($post_id);
		$dop = '';
		foreach ($ord->get_shipping_methods() as $m) {
			$dop.= $m['name'].'<br>';
		}
		echo $dop ? $dop : '&ndash;';
	}
	elseif ($column == 'order_qdl no-link')
	{
		
		$ord = wc_get_order($post_id);
		$dop = '';
		foreach ($ord->get_shipping_methods() as $m) {
			$dop.= $m['name'].'<br>';
		}
		
		if (qdl_is($dop))
		{
			echo '<span data-qdl-ajaxable="'.$post_id.'">'; // tohle se bude obnovovat při onfocus/onblur
			
			$codes = qdl_getTrackingCodes($post_id);
				
			if ($codes)
			{
				// bylo importováno
				$pickupDate = qdl_get_post_meta($post_id, '_qdl_pickup_date', true);
				
				$dir = plugins_url('qdl');
				
				$weights = qdl_get_post_meta($post_id, '_qdl_packages_weight', $single = true);
				$podbaliku = count(explode("+", $weights));
				echo '<a href="'.qdl_trackingLink($codes).'" target="_blank" title="Odkaz pre detail">'.implode(", ", $codes).'</a>';
				if ($podbaliku > 1) echo " ($podbaliku)";
				echo ' &nbsp; <a href="'.$dir.'/popup.php?action=pickuplist&pickup_date='.urlencode($pickupDate).'" onclick="qdl_popWin(this.href);return false;" class="button" title="Podací hárok" style="vertical-align:baseline;">'.date("d.m.Y", strtotime($pickupDate)).'</a>';
			}
			else {
				// poznámka řidiči
				$note = qdl_get_post_meta($post_id, '_qdl_driver_note', $single = true);
				echo '<button type="button" class="button tips qdl_driver_note" data-tip="Poznámka vodiči" data-value="'.esc_attr($note).'" style="vertical-align:baseline;">'; // styl pro starší verze
				echo '<i class="fa'.(empty($note) ? 'r' : '').' fa-sticky-note"></i>'; // @note: nutno i v JS
				echo '</button>';
				
				$defaultWeight = qdl_getConfig('default_weight');
				$hasDefaultWeight = true;
				if (!is_numeric($defaultWeight)) {
					$defaultWeight = '0';
					$hasDefaultWeight = false;
				}
				
				
				echo ' ';
				// ukládáme přes AJAX (onblur event)
				// potřebujeme znát hmotnosti (potažmo počet balíků)
				$weights = qdl_get_post_meta($post_id, '_qdl_packages_weight', $single = true);
				echo '<input type="text" data-name="qdl_packages_weight" class="tips" data-tip="HMOTNOSŤ (Kg) - U viac balíkov píšte viac hmotnosťou 1 + 5 + 2 alebo 2,5 a 2"';
				echo ' value="'.esc_attr($weights).'" placeholder="'.$defaultWeight.'" style="width:60px;';
				if ($weights == '' && !$hasDefaultWeight) echo 'background:#ffeded;'; // @note: nutno změnit taky v JS
				echo '">';
			}
			
			echo '</span>';
		}
		else {
			echo $dop ? $dop : '&ndash;';
		}
	}
}

add_filter('manage_edit-shop_order_columns', 'qdl_columns'); // starý hook #compatibility
add_action('manage_shop_order_posts_custom_column', 'qdl_column_values', 2); // starý hook #compatibility

add_filter('woocommerce_shop_order_list_table_columns', function($columns){
	return qdl_columns($columns);
});
add_action( 'woocommerce_shop_order_list_table_custom_column', function ($column, $order) {
	qdl_column_values($column, $order);
}, 10, 2);






function qdl_admin_head($x) {
	if (mb_strpos(ob_get_contents(), 'fontawesome') !== false) return; // už to někdo inkludoval
	echo '<link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.8.2/css/all.css" integrity="sha384-oS3vJWv+0UjzBfQzYUhtDYW+Pj2yciDJxpsK1OYPAYjqT085Qq/1cq5FLXAZQ7Ay" crossorigin="anonymous">';
}
add_action('admin_head', 'qdl_admin_head');




/**
* Přidáme BULK ACTIONS do výpisu objednávek + JS POUŽÍVÁM V DETAILU OBJ.
*/
function qdl_admin_list()
{
	global $pagenow, $typenow;
	
	// Ensure global variables are set (may not be necessary if you're already in the admin_footer)
    if (empty($typenow) && function_exists('get_current_screen')) {
        $screen = get_current_screen();
        $typenow = isset($screen->post_type) ? $screen->post_type : '';
    }
	
	// povolíme na stárnce pluginů a seznam objednávek
	if ($pagenow != 'plugins.php' && $typenow != 'shop_order') return;
	
	$dir = plugins_url('qdl');
	
    echo '<script type="text/javascript">';
	echo "var QDL_PLUGIN_URL = '$dir';";
	echo '</script>';
	echo '<script type="text/javascript" src="'.$dir.'/qdl.js?'.filemtime(__DIR__.'/qdl.js').'" charset="UTF-8"></script>';
}
add_action('admin_footer', 'qdl_admin_list');



/**
* SEZNAM PLUGINŮ
*/
add_filter( 'plugin_row_meta', 'qdl_plugin_fw_row_meta', 20, 4 );

if (!function_exists('qdl_plugin_fw_row_meta')) {
	/**
	* V seznamu pluginů doplníme odkaz pro NASTAVENÍ
	*/
	function qdl_plugin_fw_row_meta($plugin_meta_arr, $plugin_file, $plugin_data, $status)
	{
		if (strpos(strtolower(str_replace('\\', '/', '/'.$plugin_file)), '/qdl/init.php') !== false)
		{
			$dir = plugins_url('qdl');
			$plugin_meta_arr[2] = '<a href="https://www.qdl.sk/kontakt/" target="_blank"><span class="dashicons dashicons-admin-users"></span> Podpora / Kontakt</a>';
			$plugin_meta_arr[3] = '<a href="'.$dir.'/popup.php?action=configuration" onclick="qdl_popWin(this.href);return false;"><span class="dashicons dashicons-admin-generic"></span> Konfigurácia pluginu</a>';
		}
		
		return $plugin_meta_arr;
	}
}




/**
* TOTO DOPLNÍ TYP DOPRAVY
*/
function qdl_addshmet() {

    if ( ! class_exists( 'WC_qdl_shmet' ) ) {
        class WC_qdl_shmet extends WC_Shipping_Method {

            public function __construct( $instance_id = 0) {
                $this->id = 'qdl_shipping';
                $this->instance_id = absint( $instance_id );
                
				$this->domain = 'qdl';
				
                $this->method_title = __('QDL', $this->domain );
                $this->method_description =  __('', $this->domain );
                $this->supports = array(
                    'shipping-zones',
                    'instance-settings',
                    'instance-settings-modal',
                );
 
				$this->enabled = "yes";
                $this->init();
            }

            function init() {
                $this->init_form_fields();
                $this->init_settings();
                $this->enabled = $this->get_option( 'enabled', $this->domain );
                $this->title   = $this->get_option( 'title', $this->domain );
                $this->info    = $this->get_option( 'info', $this->domain );
                $this->cost    = $this->get_option( 'cost', $this->domain );
                add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
             }

            function init_form_fields() {
                $this->instance_form_fields = array(
                    'title' => array(
                        'type'          => 'text',
                        'title'         => __('Názov', $this->domain),
                        //'description'   => __('Title to be displayed on site.', $this->domain ),
                        'default'       => __('QDL', $this->domain ),
                    ),
                    'cost' => array(
                        'type'          => 'text',
                        'title'         => __('Cena', $this->domain),
                        //'description'   => __('Enter a cost', $this->domain ),
                        'default'       => 0,
                    ),
                );
            }

			/** bez téhle funkce se neukazuje v košíku
			*/
            public function calculate_shipping( $packages = array() ) {
                $rate = array(
                    'id'       => $this->id,
                    'label'    => $this->title,
                    'cost'     => $this->cost,
					'calc_tax' => 'per_item'
                );
                $this->add_rate($rate);
            }
        }
    }
}
add_action('woocommerce_shipping_init', 'qdl_addshmet');


function qdl_addsh( $methods ) {
    $methods['qdl_shipping'] = 'WC_qdl_shmet';
    return $methods;
}
add_filter('woocommerce_shipping_methods', 'qdl_addsh');

